#!/bin/bash
#
# Bourne Again IRC client v1.3
# author: dodo <dodo@darkwired.org>
# license: GNU GPL 
#
# changes:
# - raw mode optimalization
# - non-raw GUI mode mayor speedup
# - added GUI colors 
# - better gui parsing
# - added extra flags and commands
#

#builtin calibration:
dialog="dialog"
raw="gui"
versionn="1.3"
version="Bourne Again IRC client version $versionn - `uname` `uname -r` [`uname -m`]"


        function outputUsage()
        {
                echo "usage: "$0" [options] <server>"
		echo "options:"
		echo "	-h --help 		show this"
		echo "	-v --version		show version"
		echo "	-n --nick <nickname>	specifies nickname"
		echo "	-p --port <port>	specifies port"
		echo "	-r --raw		raw mode [fast!]"
		echo "	-f --fraw		fancy raw mode"
		echo "	-c --channel <chan>	join this channel"
		echo "	-d --nocolor		disable colors"
		echo "commands:"
		echo "	hit '/usage' too see available commands"
		exit
        }

        function outputError()
        {
                rm -Rf /tmp/bairc.`whoami`*
                echo "error: "$1
                exit
        }

        function getDialogNickname()
        {
        $dialog --title "nickname"  --inputbox "Please enter a nickname: " 0 0 `whoami` \
        2> /tmp/bairc.`whoami`.nickname
                case $? in
                        0)
                          nickname=`cat /tmp/birc.nickname`;;
                        1)
                          outputError "nickname required";;
                        255)
                          outputError "nickname required";;
                esac
        rm -Rf /tmp/birc.nickname
        }

	function parseInput()
	{
	local f=0

		if [ ${#1} == 0 ]; then
		return
		fi

		if [ $1 == "/usage" ] || [ $1 == "/h" ]; then
		echo "following commands are supported: "
		echo "	/usage 			this usage [/h]"
		echo "	/quit [msg]		quit bairc [/qt]"
		echo "	/join <channel>		join channel [/j]"
		echo "	/part <channel>		part channel [/p]"
		echo "	/talk <nickname>	talk to [/q]"
		echo "	/ver [who]		request version [/v]"
		if [ $mode != "raw" ]; then
		echo "	/raw-on			turn on  rawmode [/r1]"
		echo "	/raw-off		turn off rawmode [/r0]"
		fi
		echo "unknown commands -> server commands"
		local f=1
		fi

		if [ $1 == "/quit" ] || [ $1 == "/qt" ]; then
			if [ -z "$2" ]; then
			local quitmsg="used the "$version 
			else
			local quitmsg=$*
			fi
		ircQuit $quitmsg
		netStop
		exit
		fi

		if [ $1 == "/join" ] || [ $1 == "/j" ]; then
		ircJoin $2
		channel=$2
		local f=1
		fi

		if [ $1 == "/ver" ] || [ $1 == "/v" ]; then
			if [ -z "$2" ]; then
			local who=$channel
			else
			local who=$2
			fi
		ircGetVersion $who
		local f=1
		fi

		if [ $1 == "/part" ] || [ $1 == "/p" ]; then
		ircPart $2
		local f=1
		fi

		if [ $1 == "/nick" ] || [ $1 == "/n" ]; then
		ircNick $2
		nickname=$2
		echo $2 > /tmp/bairc.`whoami`.nick
		local f=1
		fi

		if [ $1 == "/talk" ] || [ $1 == "/q" ]; then
			if [ -z $2 ]; then
			echo "usage: /talk <nickname>"
			return
			fi
		channel=$2	
		echo "now talking to: $2"
		local f=1
		fi

		if [ $1 == "/raw-on" ] || [ $1 == "/r1" ]; then
		raw="raw"
		echo "raw" > /tmp/bairc.`whoami`.raw
		local f=1
		fi
                if [ $1 == "/raw-off" ] || [ $1 == "/r0" ]; then
                raw="gui"
		echo "gui" > /tmp/bairc.`whoami`.raw
                local f=1
                fi


		if [ ${1:0:1} == "/" ] && [ $f -eq 0 ]; then
		getTrailArgvs 1 $*	
		echo ${1:1} $ret >&666
		local f=1
		fi
		
		if [ $f -eq 0 ]; then
		ircMsg $channel $*
			if [ $mode != "raw" ]; then
			echo -en ${color[1]}
				if [ ${channel:0:1} == "#" ]; then
				echo -n "$nickname: "
				else
				echo -n "$nickname -> $channel: "
				fi
			echo -en ${color[0]}
			echo $*
			fi
		fi
		
	}

	function rawMain()
	#raw mode main listen routine
	{
	ircLogin $nickname 
        if [ ${#channel} -gt 0 ]; then
        ircJoin $channel
        fi


		while read line; do 
		ircParseLine $line
		echo "$line"
		done <&666 & 

		while read sline; do 
		parseInput $sline
		done < /dev/stdin 

	}

	function brawMain()
	#backspaced raw main listen routine
	{
        ircLogin $nickname

                while read line; do
                ircParseLine $line
                echo " "$line
                done <&666 &
        
                while read sline; do
		putBackspaces ${#sline}
                parseInput $sline
		#echo -n "["$channel"] "
                done < /dev/stdin   
                          
	}
		

	function clmMain()
	#clear mode
	{
	local dumpfile="/tmp/bairc.`whoami`.dump"
	rm -Rf /tmp/bairc.`whoami`.* 
	touch $dumpfile
        echo $raw > /tmp/bairc.`whoami`.raw
	echo $nickname > /tmp/bairc.`whoami`.nick
	chmod og-rwx /tmp/bairc.`whoami`.*

	updateTerminalData
	clmInitColors

		ircLogin $nickname
		if [ ${#channel} -gt 0 ]; then
		ircJoin $channel
		fi

	        while read line; do
                ircParseLine $line
		echo "$line" >> $dumpfile 
		clear
		updateTerminalData
		clmParseLine "$line" >> $dumpfile".gui"	
		clmOutputDumpfile $dumpfile 
		clmLowerBar
                done <&666 &

	while read sline; do
	clear 
	updateTerminalData
	parseInput $sline > $dumpfile".tmp" 
	cat $dumpfile".tmp" >> $dumpfile
        cat $dumpfile".tmp" >> $dumpfile".gui"
	#clmParseLine "$sline" >> $dumpfile".gui"
	clmOutputDumpfile $dumpfile
	clmLowerBar
	done < /dev/stdin

	}


	function clmLowerBar()
	{
        bardata="[talking to: $channel]"

		if [ $colors == "no" ]; then
		chr="-"
		else
		chr=" "
		fi

	echo -en ${color[8]}
		for ((a=0; ($x-1-${#bardata}) > a; a++))
		do
		echo -n "$chr" 
		done
		echo -n "$bardata"
		echo -n "$chr"
	echo -e ${color[0]}
	local nnfile="/tmp/bairc.`whoami`.nick"
	echo -n "[`cat $nnfile`] "
	}

	function clmOutputDumpfile()
	{
	rawfile="/tmp/bairc.`whoami`.raw"

		if [ `cat $rawfile` == "gui" ]; then
		#tailfile="/tmp/bairc.`whoami`.tailfile"
		#tail -n `expr $y - 2` $dumpfile > $tailfile
			#while read l; do
			#clmParseLine "$l"
			#done < $tailfile 
		tail -n `expr $y - 2` $dumpfile".gui"
		else
		tail -n `expr $y - 2` $dumpfile
		fi


	}

		
	function clmParseLine()
	{
		clmParseLiner $1
		if [ $ret -eq 0 ]; then
                        if [ ${1:0:1} == ":" ]; then
                        getTrailArgvs 3 $*
                        echo "] " $ret
                        else
                        echo "$1" 
                        fi

		fi
	}

	function clmParseLiner()
	{
	ret=0
                if [ $2 == "PRIVMSG" ]; then
                getTrailArgvs 3 $*
                local msg=$ret
                getNickByHostmask $1
                #echo -en ${color[1]}
			local nnfile="/tmp/bairc.`whoami`.nick"
			if [ $3 == `cat $nnfile` ]; then
			echo -n "<"$ret" -> "$3">"
			else
			echo -n "<"$ret">"
			fi
		#echo -en ${color[0]} 
		echo " "${msg:2}
                unset msg
                ret=1
                fi

                if [ $2 == "NOTICE" ]; then
                getTrailArgvs 3 $*
                local msg=$ret
                getNickByHostmask $1
		echo -en ${color[2]}
                echo -n "-"$ret"-" ${msg:1}
		echo -e ${color[0]}
                unset msg
                ret=1
                fi

		if [ $2 == "JOIN" ]; then
		getNickByHostmask $1
		echo -en ${color[5]}
		echo -n "-> $ret has joined [${1:1}]"
		echo -e ${color[0]}
		ret=1
		fi

                if [ $2 == "PART" ]; then
                getNickByHostmask $1
                echo -en ${color[5]}
                echo -n "<- $ret has left [${1:1}]"
                echo -e ${color[0]}
                ret=1
                fi


	}
	
	function clmInitColors()
	#note: http://www.linuxgazette.com/issue65/padala.html
	{
		if [ $colors == "no" ]; then
		return
		fi
	color[0]='\e[0;39m\e[0;49m'	#default
	color[1]='\e[1m'		#bold/bright
	color[2]='\e[0;31m'		#red
	color[3]='\e[0;32m'             #green
	color[4]='\e[0;33m'		#yellow
	color[5]='\e[0;34m'		#blue

	color[8]='\e[0;30;42m'		#lowerbar
		
	



	}

	function updateTerminalData()
	{
	x=`tput cols`
	y=`tput lines`
		if [ ${#x} == 0 ] || [ ${#y} == 0 ]; then
		outputError "updateTerminalData error"
		fi
	}

        function getTrailArgvs()
        {
        local armv
                c=0
                for arg in ${@}; do
                        if [ $c -gt $1 ]; then
                        armv[$c]=$arg
                        fi
                let c=c+1
                done
        ret="${armv[@]}"
        }

	function putBackspaces()
	{

		for ((a=0; $1 > a; a++))
		do
		echo -en '\x08'
		done
	}

	function getStrPos()
	{
		for ((i=0; ${#1} > i ; i++))
		do
			if [ ${1:$i:1} == $2 ]; then
			ret=$i
			break
			fi
		done
	}
	
	function getNickByHostmask()
	{
	getStrPos $1 "!"
        ret=${1:1:($ret-1)}
	}


# -- network I/O functions
	
        function netStart()
        {
        exec 666<> /dev/tcp/$server/$port
	}
        

	function netStop()
	{
	exec 666>&-
	}

# -- IRC subroutines

	function ircParseLine()
	{
		if [ $1 == "PING" ]; then
		ircPong $2
		fi

		if [ $1 == "ERROR" ]; then
		outputError "$*"
		fi
		

		if [ $2 == "PRIVMSG" ]; then
		local argv=`echo $4 | tr -d [:space:]`

			if [ ${argv:2:7} == "VERSION" ]; then
			ircSendVersion $1
			fi

		fi

	}


	function ircPong()
	{
	echo "PONG $1" >&666
	}

	function ircLogin()
	#log on to the irc server
	{
	echo "USER $nickname $nickname $nickname $nickname" >&666
	echo "NICK $nickname" >&666
	}

	function ircQuit()
	{
	echo "QUIT :$*" >&666
	}

	function ircJoin()
	{
	echo "JOIN $1" >&666
	}

	function ircSendVersion()
	#thanks to rainman's torc: rainman@darkwired.org
	{
	getNickByHostmask $1
	echo "NOTICE $ret :"`echo -en '\x01'`"VERSION $version"`echo -en '\x01'` >&666
	}

	function ircGetVersion()
	#thanks to rainman's torc: rainman@darkwired.org
	{
	echo "PRIVMSG $1 :"`echo -en '\x01'`"VERSION"`echo -en '\x01'`"" >&666
	}

	function ircPart()
	{
	echo "PART $1" >&666
	}

	function ircMsg()
	{
	armv=""
		c=0     
        	for arg in ${@}; do
		if [ $c -gt 0 ]; then 
        	armv[$c]="$arg"
		fi
        	let c=c+1
       	 	done

	echo "PRIVMSG $1 :${armv[@]:1}" >&666
	unset armv
	}

	function ircNick()
	{
	echo "NICK $1" >&666
	}
	
# -- general option parsing 


	c=0
	for arg in ${@}; do
	argv[$c]=$arg
	let c=c+1
	done

	if [ ${#} -lt 1 ]; then
	outputUsage $0
	fi

	o=0
	mode="gui"
	colors="yes"
	for ((i=0; ${#argv[@]} > i ; i++))
	do
	argument=${argv[$i]}
	#echo "arg: "$argument

		if [ $argument == "-h" ] || [ $argument == "--help" ]; then
		outputUsage $0
		fi

                if [ $argument == "-v" ] || [ $argument == "--version" ]; then
		echo $versionn; exit
                fi

		if [ $argument == "-n" ] || [ $argument == "--nick" ]; then
		nickname=${argv[($i+1)]}
		#let o=o+1
		fi

		if [ $argument == "-c" ] || [ $argument == "--channel" ]; then
		channel=${argv[($i+1)]}
		#let o=o+1
		fi

		if [ $argument == "-p" ] || [ $argument == "--port" ]; then	
		port=${argv[($i+1)]}
		#let o=o+1
		fi

		if [ $argument == "-r" ] || [ $argument == "--raw" ]; then
		mode="raw"
		fi

		if [ $argument == "-d" ] || [ $argument == "--nocolors" ]; then
		colors="no"
		fi 

                if [ $argument == "-f" ] || [ $argument == "--fraw" ]; then
                raw="raw"
                fi


	let o=o+1
	done
	
		if [ ${#nickname} -eq 0 ]; then
		nickname=`whoami`
		fi

		if [ ${#port} -eq 0 ]; then
		port=6667
		fi

		server=${argv[($o-1)]}
		if [ ${#server} -eq 0 ]; then 
		outputUsage $0
		fi


	if [ $server == "-c" ]; then
	echo "try to escape the channel parameter '-c \#channel'"
	exit
	fi

	netStart
		if [ $mode == "raw" ]; then
		rawMain
		else
		clmMain
		fi
	netStop


